#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <unistd.h>
#include <getopt.h>

#include "mpi_interfaces.h"

/**********************************************************************************************/
/* Load the routing Data structures from the RoutingFile. The RoutingFile contains a collection of data structures 
    which contain SRIO routing table which can be loaded in to the memory before invoking other APIs. */
/***********************************************************************************************/
void* SrioRoutingStructuresCreate(char *configFile_str/*IN*/) 
{
     FILE *configFile_ptr;
     configFile_ptr= fopen(configFile_str,"rb");
     if(configFile_ptr==NULL) {
         printf("\n SrioRoutingStructuresCreate: Unable to open file %s", configFile_str);
         return NULL; 
     } else {
         return(srioUtils_LoadFromFile(configFile_ptr)); 
      }
}
/* Get the SRIO ID from the physical location (eg C1N3 -> 3). Returns Status (SUCCESS/FAIL) along with the srioID.
 */
int SrioGetSrioIDFromPhysID(void *base  /*IN*/, int cartridge_id  /*IN*/, int node_id /*IN*/,unsigned short *srioID /*OUT*/)
{
     return(srioUtils_map_physID_to_SRIOId(base,cartridge_id,node_id,srioID));
}

/* Gets the Physical ID from the SRIO ID (eg 3 -> C1N3). Returns Status (SUCCESS/FAIL) along with the physID */ 
int SrioGetPhysIDFromSRIOID(void *base /*IN*/, unsigned short srioID, int *cartridge_id/*OUT*/,int *node_id/*OUT*/)
{
    return(srioUtils_map_SRIOId_to_physID(base,srioID,cartridge_id,node_id));
}

/* Return the physical port to route SRIO packets originating from srioID_src to srioID_dest */
int SrioGetRoutingPort(void *base /*IN*/, unsigned short srioID_src  /*IN*/, unsigned short srioID_dest  /*IN*/)
{
     return(srioUtils_find_routing_port(base,srioID_src,srioID_dest));
}

/* Create a routing Table from a cluster with a set of physical cartridge IDs*/
int SrioCreateRoutingTableFromCluster(void *base, int cartridge_phys_ids[]/*IN*/, int num_cartridge_phys_ids/*IN*/)
{
    return(srioUtils_formCluster(base,cartridge_phys_ids,num_cartridge_phys_ids));
}

/* Display cluster information */
int SrioDisplayClusterInformation(void *base) 
{
    return(srioUtils_displayCluster(base));
}

/* Reset cluster information */
int SrioResetCluster(void *base) 
{
    return(srioUtils_ResetCluster(base));
}

#ifdef DEVICE_K2H 
/* Reset packet forwarding registers */
int SrioResetPktFwdingRegs(void *base, void *srio_base_virtual, int srio_id)
{
    return(srioUtils_resetFwdingRegs(base,srio_base_virtual,srio_id));
}

/* Program packet forwarding registers */
int SrioProgramPktFwdingRegs(void *base, void *srio_base_virtual, int srio_id) 
{
    return(srioUtils_progFwdingRegs(base,srio_base_virtual,srio_id));
}

/* Display packet forwarding registers */
int SrioDisplayPktFwdingRegs(void *base, void *srio_base_virtual, int srio_id)
{
    return(srioUtils_displayPortForwardingRegs(base,srio_base_virtual,srio_id));
}
#endif

/* Create list of hostnames */
void *SrioHostnameListCreate (char *file_hostname) {
  int i,j,inLen;
  char lineIn[320];
  SrioHostList_t *srio_list;
  FILE *fin = fopen(file_hostname, "rt");
  char delims[] = " #\n";
  char *result = NULL;

  if(fin == NULL) return NULL;

  srio_list = (SrioHostList_t *)calloc (sizeof(SrioHostList_t), 33);
  for (i = 0; i < 32; ) {
     if(fgets(lineIn, 320, fin) == NULL) break;
     inLen = strlen(lineIn);
     if(inLen < 6) continue;
     if(lineIn[0] == '#') continue;
     lineIn[inLen - 1] = 0;
     result = strtok( lineIn, delims );
     if(result == NULL) break;
     srio_list[i].cartridge_id = atoi(result);
     result = strtok( NULL, delims );
     if(result == NULL) break;
     srio_list[i].node_id = atoi(result);
     result = strtok( NULL, delims );
     if(result == NULL) break;
     strcpy(srio_list[i].hostname, result);
     i ++;
  }
#ifdef BTL_MPI_INTERFACES_VERBOSE
  printf ("\nTotal SRIO Hostname list in %s:%d\n", file_hostname, i); fflush(stdout);
  for (j = 0; j < i; j ++) {
    printf ("[%2d] %s c%dn%d\n", j, srio_list[j].hostname, srio_list[j].cartridge_id, srio_list[j].node_id);
  }
#endif
  return srio_list;
}

/* Return the physical location (cartridge_id and node_id) using hostname */
int SrioHostname2PhysID (void *hostnameList, char *hostname, int *cartridge_id, int *node_id)
{
int i, hostname_len;
SrioHostList_t *srio_list = (SrioHostList_t *)hostnameList;
  hostname_len = strlen(hostname);
  for (i = 0; i < 32; i ++)
  {
     if((srio_list[i].cartridge_id == 0) && (srio_list[i].node_id == 0)) break;
     if(strcmp(hostname, srio_list[i].hostname) == 0)
     {
        *cartridge_id = srio_list[i].cartridge_id;
        *node_id = srio_list[i].node_id;
        return i;
     }
  }
  return -1;
}
/** nothing past this point **/


